from django import template
from django.conf import settings
from feincms3.templatetags.feincms3 import translations


register = template.Library()


@register.filter
def site_translations(page):
    # Cannot use page.translations().active() because .active() will always
    # filter out pages from other sites.
    try:
        pages = page.translations().filter(is_active=True)
    except (AttributeError, TypeError):
        pages = None
    return [
        {
            **item,
            "site": settings.SITES[item["code"]],
            "site_link": "//" + settings.SITES[item["code"]]["host"],
        }
        for item in translations(pages)
    ]



from content_editor.models import Region, create_plugin_base
from django.utils.translation import gettext_lazy as _
from feincms3 import plugins
from feincms3.applications import ApplicationType, PageTypeMixin, TemplateType

from feincms3_language_sites.models import AbstractPage


class Page(AbstractPage, PageTypeMixin):
    TYPES = [
        TemplateType(
            key="standard",
            title=_("standard"),
            template_name="pages/standard.html",
            regions=(Region(key="main", title=_("Main")),),
        ),
        TemplateType(
            key="with-sidebar",
            title=_("with sidebar"),
            template_name="pages/with-sidebar.html",
            regions=(
                Region(key="main", title=_("Main")),
                Region(key="sidebar", title=_("Sidebar")),
            ),
        ),
        ApplicationType(
            key="application",
            title=_("application"),
            urlconf="testapp.application_urls",
        ),
    ]


PagePlugin = create_plugin_base(Page)


class Snippet(plugins.snippet.Snippet, PagePlugin):
    TEMPLATES = [("snippet.html", _("snippet"))]



import os


DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}}
DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.admin",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.staticfiles",
    "django.contrib.messages",
    "testapp",
    # Libraries
    "feincms3",
    "content_editor",
    "tree_queries",
]

MEDIA_ROOT = "/media/"
STATIC_URL = "/static/"
BASEDIR = os.path.dirname(__file__)
MEDIA_ROOT = os.path.join(BASEDIR, "media/")
STATIC_ROOT = os.path.join(BASEDIR, "static/")
SECRET_KEY = "supersikret"
LOGIN_REDIRECT_URL = "/?login=1"
ALLOWED_HOSTS = ["*"]

ROOT_URLCONF = "testapp.urls"
LANGUAGES = (("en", "English"), ("de", "German"), ("fr", "French"))

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    # "django.middleware.locale.LocaleMiddleware",
    "feincms3_language_sites.middleware.site_middleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

# NOTE! This isn't a valid setting.
SITES = {
    "en": {},
    "de": {},
    "fr": {},
}



from django.conf.urls.i18n import i18n_patterns
from django.contrib import admin
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render
from django.urls import include, path, re_path

from testapp import views


pages_urlpatterns = (
    [
        path("", lambda request: HttpResponseRedirect(f"/{request.LANGUAGE_CODE}/")),
        re_path(r"^(?P<path>[-\w/]+)/$", views.page_detail, name="page"),
        path("", views.page_detail, name="root"),
    ],
    "pages",
)


urlpatterns = i18n_patterns(
    path("i18n/", lambda request: HttpResponse(request.LANGUAGE_CODE))
) + [
    re_path(r"^admin/", admin.site.urls),
    path("404/", lambda request: render(request, "404.html")),
    path("", include(pages_urlpatterns)),
]



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.test import TestCase
from django.test.utils import isolate_apps, override_settings
from django.urls import NoReverseMatch
from django.utils.translation import override

from feincms3_language_sites.checks import check_sites
from feincms3_language_sites.models import (
    CACHE_KEY,
    AbstractPage,
    apps_urlconfs,
    cache,
    reverse_language_site_app,
    site_for_host,
)
from feincms3_language_sites.templatetags.feincms3_language_sites import (
    site_translations,
)
from testapp.models import Page


class SiteMiddlewareTest(TestCase):
    @override_settings(SITES={"de": {"host": "testserver"}})
    def test_no_400(self):
        page = Page.objects.create(
            title="home",
            slug="home",
            path="/de/",
            static_path=True,
            language_code="de",
            is_active=True,
        )
        self.assertContains(self.client.get("/de/"), "home - testapp")
        self.assertEqual(page.get_absolute_url(), "http://testserver/de/")

    @override_settings(SITES={"de": {"host": "testserver2"}})
    def test_400(self):
        Page.objects.create(
            title="home",
            slug="home",
            path="/de/",
            static_path=True,
            language_code="de",
            is_active=True,
        )
        self.assertEqual(self.client.get("/de/").status_code, 400)

    @override_settings(SITES={"de": {"host": "blub.example.com"}})
    def test_case_insensitive(self):
        page = Page.objects.create(
            title="home",
            slug="home",
            path="/de/",
            static_path=True,
            language_code="de",
            is_active=True,
        )
        response = self.client.get("/de/", headers={"host": "Blub.Example.Com"})
        self.assertContains(response, "home - testapp")
        self.assertEqual(page.get_absolute_url(), "http://blub.example.com/de/")


@override_settings(
    MIDDLEWARE=settings.MIDDLEWARE
    + ["feincms3_language_sites.middleware.redirect_to_site_middleware"],
    SITES={
        "de": {"host": "de.example.com"},
        "fr": {"host": "fr.example.com"},
    },
)
class PortHandlingTest(TestCase):
    def setUp(self):
        Page.objects.create(
            title="home",
            slug="home",
            path="/de/",
            static_path=True,
            language_code="de",
            is_active=True,
        )

    def test_site_for_host_strips_port_80(self):
        with override_settings(SITES={"de": {"host": "de.example.com"}}):
            self.assertEqual(
                site_for_host("de.example.com:80"),
                {"host": "de.example.com", "language_code": "de"},
            )

    def test_site_for_host_strips_port_443(self):
        with override_settings(SITES={"de": {"host": "de.example.com"}}):
            self.assertEqual(
                site_for_host("de.example.com:443"),
                {"host": "de.example.com", "language_code": "de"},
            )

    def test_site_for_host_strips_port_80_with_host_re(self):
        with override_settings(
            SITES={"de": {"host": "de.example.com", "host_re": r"de\.example\.com"}}
        ):
            result = site_for_host("de.example.com:80")
            self.assertIsNotNone(result)
            self.assertEqual(result["host"], "de.example.com")

    def test_site_for_host_keeps_nonstandard_port(self):
        with override_settings(SITES={"de": {"host": "de.example.com"}}):
            self.assertIsNone(site_for_host("de.example.com:8080"))

    def test_middleware_port_80(self):
        response = self.client.get("/de/", headers={"host": "de.example.com:80"})
        self.assertContains(response, "home - testapp")

    def test_middleware_port_443(self):
        response = self.client.get(
            "/de/", headers={"host": "de.example.com:443"}, secure=True
        )
        self.assertContains(response, "home - testapp")

    def test_middleware_port_80_no_redirect(self):
        response = self.client.get("/de/", headers={"host": "de.example.com:80"})
        self.assertEqual(response.status_code, 200)

    def test_middleware_port_443_no_redirect(self):
        response = self.client.get(
            "/de/", headers={"host": "de.example.com:443"}, secure=True
        )
        self.assertEqual(response.status_code, 200)


@override_settings(
    MIDDLEWARE=settings.MIDDLEWARE
    + ["feincms3_language_sites.middleware.redirect_to_site_middleware"],
    SITES={
        "de": {
            "host": "de.example.com",
            "host_re": r"de.example.com|testserver",
        },
        "fr": {"host": "fr.example.com"},
    },
)
class RedirectMiddlewareTest(TestCase):
    def test_redirect(self):
        Page.objects.create(
            page_type="standard",
            title="de",
            slug="de",
            language_code="de",
            is_active=True,
        )
        Page.objects.create(
            page_type="standard",
            title="fr",
            slug="fr",
            language_code="fr",
            is_active=True,
        )

        response = self.client.get("/de/", headers={"host": "de.example.com"})
        self.assertContains(response, "<h1>de</h1>")

        response = self.client.get("/de/", headers={"host": "fr.example.com"})
        self.assertEqual(response.status_code, 404)

        response = self.client.get("/de/")
        self.assertEqual(response.status_code, 301)
        self.assertEqual(response["Location"], "http://de.example.com/de/")

        with override_settings(DEBUG=True):
            response = self.client.get("/de/")
            self.assertEqual(response.status_code, 302)
            self.assertEqual(response["Location"], "http://de.example.com/de/")

        with override_settings(SECURE_SSL_REDIRECT=True):
            response = self.client.get("/de/", secure=False)
            self.assertEqual(response.status_code, 301)
            self.assertEqual(response["Location"], "https://de.example.com/de/")

            response = self.client.get("/de/", secure=True)
            self.assertEqual(response.status_code, 301)
            self.assertEqual(response["Location"], "https://de.example.com/de/")

            response = self.client.get(
                "/de/", headers={"host": "de.example.com"}, secure=False
            )
            self.assertEqual(response.status_code, 301)
            self.assertEqual(response["Location"], "https://de.example.com/de/")

            response = self.client.get(
                "/de/", headers={"host": "de.example.com"}, secure=True
            )
            self.assertEqual(response.status_code, 200)

        with override_settings(SECURE_SSL_REDIRECT=False):
            response = self.client.get("/de/", secure=False)
            self.assertEqual(response.status_code, 301)
            self.assertEqual(response["Location"], "http://de.example.com/de/")

            response = self.client.get("/de/", secure=True)
            self.assertEqual(response.status_code, 301)
            self.assertEqual(response["Location"], "https://de.example.com/de/")

            response = self.client.get(
                "/de/", headers={"host": "de.example.com"}, secure=False
            )
            self.assertEqual(response.status_code, 200)

            response = self.client.get(
                "/de/", headers={"host": "de.example.com"}, secure=True
            )
            self.assertEqual(response.status_code, 200)

        # self.assertContains(response, "<h1>de</h1>")
        # print(response, response.content.decode("utf-8"))


@override_settings(
    MIDDLEWARE=["feincms3_language_sites.middleware.redirect_to_site_middleware"]
    + settings.MIDDLEWARE,
    SITES={"de": {"host": "de.example.com"}},
)
class WrongOrderRedirectMiddlewareTest(TestCase):
    def test_redirect(self):
        Page.objects.create(
            page_type="standard",
            title="de",
            slug="de",
            language_code="de",
            is_active=True,
        )

        with self.assertRaises(ImproperlyConfigured):
            self.client.get("")


@override_settings(
    SITES={
        "en": {"host": "en.example.com"},
        "de": {"host": "de.example.com"},
        "fr": {"host": "fr.example.com"},
    },
)
class ReverseAppTest(TestCase):
    def setUp(self):
        cache.delete(CACHE_KEY)

    def test_reverse_app(self):
        Page.objects.create(
            page_type="application",
            title="de",
            slug="de",
            language_code="de",
            is_active=True,
        )
        Page.objects.create(
            page_type="application",
            title="fr",
            slug="fr",
            language_code="fr",
            is_active=True,
        )

        with self.assertNumQueries(1):
            urlconf_map = apps_urlconfs()
        self.assertEqual(
            urlconf_map,
            {
                "en": "testapp.urls",
                "de": "urlconf_fd5a0537769ae95be140eeccf96b8d39",
                "fr": "urlconf_5a381ea1f57c9d934cf1fac7c23956c0",
            },
        )

        with self.assertNumQueries(0):
            with override("de"):
                url = reverse_language_site_app("application", "root")
                self.assertEqual(url, "http://de.example.com/de/")

                url = reverse_language_site_app("does-not-exist", "view", fallback="/")
                self.assertEqual(url, "http://de.example.com/")

            with override("fr"):
                url = reverse_language_site_app("application", "root")
                self.assertEqual(url, "http://fr.example.com/fr/")

            with override("en"), self.assertRaises(NoReverseMatch):
                reverse_language_site_app("application", "root")

    def test_invalid_language(self):
        # "it" doesn't exist in LANGUAGES
        Page.objects.create(
            page_type="application",
            title="it",
            slug="it",
            language_code="it",
            is_active=True,
        )
        self.assertEqual(
            apps_urlconfs(),
            {"en": "testapp.urls", "de": "testapp.urls", "fr": "testapp.urls"},
        )


class ChecksTest(TestCase):
    def assertCheckCodes(self, errors, codes):  # noqa: N802
        self.assertCountEqual({error.id for error in errors}, codes)

    @override_settings()
    def test_missing(self):
        del settings.SITES

        self.assertCheckCodes(
            check_sites(None),
            {"feincms3_language_sites.E001"},
        )

    @override_settings(
        SITES={"it": {"host": "it.example.com"}},
    )
    def test_mismatch(self):
        self.assertCheckCodes(
            check_sites(None),
            {"feincms3_language_sites.E002", "feincms3_language_sites.E003"},
        )


@override_settings(
    SITES={
        "en": {"host": "en.example.com"},
        "de": {"host": "de.example.com"},
        "fr": {"host": "fr.example.com"},
    },
)
class PagesModelTest(TestCase):
    def test_validation(self):
        page_de = Page.objects.create(
            title="home",
            slug="home",
            path="/",
            static_path=True,
            language_code="de",
            is_active=True,
        )
        page_en = Page.objects.create(
            title="home",
            slug="home",
            path="/",
            static_path=True,
            language_code="en",
            is_active=True,
        )

        # Doesn't raise
        page_de.full_clean()
        page_en.full_clean()

        with self.assertRaises(ValidationError):
            page_en.language_code = "de"
            page_en.full_clean()

    def test_site_translations_filter(self):
        page_en = Page.objects.create(
            title="home",
            slug="home",
            path="/home/",
            static_path=True,
            language_code="en",
            is_active=True,
        )
        page_de = Page.objects.create(
            title="start",
            slug="start",
            path="/start/",
            static_path=True,
            language_code="de",
            is_active=True,
            translation_of=page_en,
        )

        self.assertEqual(
            site_translations(page_de),
            [
                {
                    "code": "en",
                    "name": "English",
                    "object": page_en,
                    "site": {"host": "en.example.com"},
                    "site_link": "//en.example.com",
                },
                {
                    "code": "de",
                    "name": "German",
                    "object": page_de,
                    "site": {"host": "de.example.com"},
                    "site_link": "//de.example.com",
                },
                {
                    "code": "fr",
                    "name": "French",
                    "object": None,
                    "site": {"host": "fr.example.com"},
                    "site_link": "//fr.example.com",
                },
            ],
        )

        self.assertEqual(
            [lang["code"] for lang in site_translations(None)],
            ["en", "de", "fr"],
        )
        self.assertEqual(
            [lang["code"] for lang in site_translations("page")],
            ["en", "de", "fr"],
        )

    @isolate_apps("testapp")
    def test_page_with_missing_unique_together(self):
        """Page subclass without unique_together fails validation"""

        class Page(AbstractPage):
            class Meta:
                pass

        errors = Page.check()
        error_ids = [error.id for error in errors]
        self.assertIn("feincms3_language_sites.E001", error_ids)



from django.shortcuts import get_object_or_404, render
from feincms3.renderer import RegionRenderer, render_in_context

from testapp.models import Page, Snippet


renderer = RegionRenderer()
renderer.register(
    Snippet,
    lambda plugin, context: render_in_context(
        context, plugin.template_name, {"additional": "context"}
    ),
)


def page_detail(request, path=None):
    page = get_object_or_404(Page.objects.active(), path=(f"/{path}/") if path else "/")
    page.activate_language(request)

    return render(
        request,
        page.type.template_name,
        {
            "page": page,
            "regions": renderer.regions_from_item(
                page, inherit_from=page.ancestors().reverse()
            ),
        },
    )



import re
import sys

from django.conf import settings
from django.core.cache import cache
from django.core.checks import Error
from django.core.validators import RegexValidator
from django.db import models
from django.utils.translation import get_language, gettext_lazy as _
from feincms3 import applications, pages
from feincms3.applications import apps_urlconf, reverse_app
from feincms3.mixins import LanguageAndTranslationOfMixin

import feincms3_language_sites.checks  # noqa: F401


def _protocol():
    return "https:" if settings.SECURE_SSL_REDIRECT else "http:"


def _strip_default_port(host):
    """
    Strip port 80 and 443 from a host string since they are the default ports
    for HTTP and HTTPS respectively and are often omitted in site configurations.
    """
    if host.startswith("["):
        # IPv6 address in brackets, e.g. [::1]:80
        if host.endswith((":80", ":443")):
            return host.rsplit(":", 1)[0]
    elif ":" in host:
        domain, port = host.rsplit(":", 1)
        if port in ("80", "443"):
            return domain
    return host


def site_for_host(host):
    host = _strip_default_port(host)
    for language_code, site in settings.SITES.items():
        site.setdefault("language_code", language_code)
        if "host_re" in site:
            if re.search(site["host_re"], host, re.I):
                return site
        elif site["host"].lower() == host.lower():
            return site
    return None


CACHE_KEY = "reverse-language-site-app-cache"
CACHE_TIMEOUT = 60


def apps_urlconfs():
    urlconf_map = cache.get(CACHE_KEY)
    if not urlconf_map or any(
        module for module in urlconf_map.values() if module not in sys.modules
    ):
        fields = ("path", "page_type", "app_namespace", "language_code")
        apps = {code: [] for code, name in settings.LANGUAGES}
        for app in (
            applications._APPS_MODEL._default_manager.filter(is_active=True)
            .without_tree_fields()
            .exclude(app_namespace="")
            .values_list(*fields)
            .order_by(*fields)
        ):
            # Skip apps with languages not in LANGUAGES
            if app[-1] not in apps:
                continue
            apps[app[-1]].append(app)

        urlconf_map = {
            code: apps_urlconf(apps=site_apps) for code, site_apps in apps.items()
        }
        cache.set(CACHE_KEY, urlconf_map, timeout=CACHE_TIMEOUT)
    return urlconf_map


def reverse_language_site_app(*args, **kwargs):
    language_code = get_language()
    kwargs["urlconf"] = apps_urlconfs()[language_code]
    url = reverse_app(*args, **kwargs, languages=[language_code])
    host = settings.SITES[language_code]["host"]
    return f"{_protocol()}//{host}{url}"


class AbstractPageQuerySet(pages.AbstractPageQuerySet):
    def active(self, *, language_code=None):
        return self.filter(
            is_active=True, language_code=language_code or get_language()
        )


class AbstractPage(pages.AbstractPage, LanguageAndTranslationOfMixin):
    # Exactly the same as BasePage.path,
    # except that it is not unique:
    path = models.CharField(
        _("path"),
        max_length=1000,
        blank=True,
        help_text=_(
            "Automatically generated by concatenating the parent's path and"
            " the slug unless the path is defined manually."
        ),
        validators=[
            RegexValidator(
                regex=r"^/(|.+/)$",
                message=_("Path must start and end with a slash (/)."),
            )
        ],
    )

    objects = AbstractPageQuerySet.as_manager(with_tree_fields=True)

    class Meta(pages.AbstractPage.Meta):
        abstract = True
        unique_together = [
            ("language_code", "path"),
            ("language_code", "translation_of"),
        ]

    @classmethod
    def check(cls, **kwargs):
        errors = super().check(**kwargs)
        errors.extend(cls._check_feincms3_language_sites_page(**kwargs))
        return errors

    @classmethod
    def _check_feincms3_language_sites_page(cls, **kwargs):
        unique_together = [set(fields) for fields in cls._meta.unique_together]
        if {"language_code", "path"} not in unique_together:
            yield Error(
                "Models using the feincms3-language-sites page must ensure"
                " that paths exist only once per language.",
                obj=cls,
                id="feincms3_language_sites.E001",
                hint='Add ("language_code", "path") to unique_together.',
            )

    def _set_parent(self, parent):
        self.parent = parent
        self.language_code = parent.language_code if parent else self.language_code

    def _clash_candidates(self):
        return super()._clash_candidates().filter(language_code=self.language_code)

    @property
    def site(self):
        return settings.SITES[self.language_code]

    def get_absolute_url(self):
        host = self.site["host"]
        url = super().get_absolute_url()
        return f"{_protocol()}//{host}{url}"



from django.conf import settings
from django.core.exceptions import DisallowedHost, ImproperlyConfigured
from django.http import HttpResponsePermanentRedirect, HttpResponseRedirect
from django.utils import translation


def site_middleware(get_response):
    from feincms3_language_sites.models import site_for_host  # noqa: PLC0415

    def middleware(request):
        request.site = site_for_host(request.get_host())
        if request.site is None:
            raise DisallowedHost(f"No configuration found for {request.get_host()!r}")

        translation.activate(request.site["language_code"])
        request.LANGUAGE_CODE = translation.get_language()
        response = get_response(request)
        response.setdefault("Content-Language", translation.get_language())
        return response

    return middleware


def redirect_to_site_middleware(get_response):
    from feincms3_language_sites.models import _strip_default_port  # noqa: PLC0415

    def middleware(request):
        if not hasattr(request, "site"):
            raise ImproperlyConfigured(
                'No "site" attribute on request. Insert site_middleware'
                " before redirect_to_site_middleware."
            )

        # Host matches, and either no HTTPS enforcement or already HTTPS
        # Do a redirect if
        # - Host doesn't match
        # - We should be on HTTPS but are not
        if _strip_default_port(request.get_host()) != request.site["host"] or (
            settings.SECURE_SSL_REDIRECT and not request.is_secure()
        ):
            if settings.SECURE_SSL_REDIRECT or request.is_secure():
                protocol = "https"
            else:
                protocol = "http"
            redirect_class = (
                HttpResponseRedirect
                if settings.DEBUG
                else HttpResponsePermanentRedirect
            )
            return redirect_class(
                f"{protocol}://{request.site['host']}{request.get_full_path()}"
            )

        return get_response(request)

    return middleware



from django.conf import settings
from django.core.checks import Error, register


@register
def check_sites(app_configs, **kwargs):
    if not hasattr(settings, "SITES"):
        return [
            Error(
                "The SITES setting is missing.",
                id="feincms3_language_sites.E001",
            )
        ]

    errors = []

    languages = {code for code, name in settings.LANGUAGES}
    sites = set(settings.SITES)

    if sites_invalid_language_codes := sites - languages:
        errors.append(
            Error(
                f"SITES contains language codes which do not exist in LANGUAGES:"
                f" {', '.join(sorted(sites_invalid_language_codes))}",
                id="feincms3_language_sites.E002",
            )
        )

    if missing_sites := languages - sites:
        errors.append(
            Error(
                f"LANGUAGES contains language codes which do not exist in SITES:"
                f" {', '.join(sorted(missing_sites))}",
                id="feincms3_language_sites.E003",
            )
        )

    return errors



# Generated by Django 3.2.7 on 2021-09-13 15:40

import django.core.validators
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Page",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        choices=[("", "")],
                        default="en",
                        max_length=10,
                        verbose_name="language",
                    ),
                ),
                (
                    "page_type",
                    models.CharField(
                        choices=[("", "")],
                        default="standard",
                        max_length=100,
                        verbose_name="page type",
                    ),
                ),
                (
                    "app_namespace",
                    models.CharField(
                        blank=True,
                        editable=False,
                        max_length=100,
                        verbose_name="app instance namespace",
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(default=True, verbose_name="is active"),
                ),
                ("title", models.CharField(max_length=200, verbose_name="title")),
                ("slug", models.SlugField(verbose_name="slug")),
                (
                    "position",
                    models.PositiveIntegerField(
                        db_index=True,
                        editable=False,
                        validators=[
                            django.core.validators.MinValueValidator(
                                limit_value=1,
                                message="Position is expected to be greater than zero.",
                            )
                        ],
                    ),
                ),
                (
                    "static_path",
                    models.BooleanField(
                        default=False, verbose_name="manually define the path"
                    ),
                ),
                (
                    "path",
                    models.CharField(
                        blank=True,
                        help_text="Generated automatically if 'static path' is unset.",
                        max_length=1000,
                        validators=[
                            django.core.validators.RegexValidator(
                                message="Path must start and end with a slash (/).",
                                regex="^/(|.+/)$",
                            )
                        ],
                        verbose_name="path",
                    ),
                ),
                (
                    "parent",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="children",
                        to="testapp.page",
                        verbose_name="parent",
                    ),
                ),
                (
                    "translation_of",
                    models.ForeignKey(
                        blank=True,
                        limit_choices_to={"language_code": "en"},
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="testapp.page",
                        verbose_name="translation of",
                    ),
                ),
            ],
            options={
                "verbose_name": "page",
                "verbose_name_plural": "pages",
                "ordering": ["position"],
                "abstract": False,
                "unique_together": {("language_code", "path")},
            },
        ),
        migrations.CreateModel(
            name="Snippet",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "template_name",
                    models.CharField(
                        choices=[("", "")], max_length=100, verbose_name="template"
                    ),
                ),
                ("region", models.CharField(max_length=255)),
                ("ordering", models.IntegerField(default=0)),
                (
                    "parent",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="testapp_snippet_set",
                        to="testapp.page",
                    ),
                ),
            ],
            options={
                "verbose_name": "predefined snippet",
                "verbose_name_plural": "predefined snippets",
                "abstract": False,
            },
        ),
    ]



from content_editor.admin import ContentEditor
from django.contrib import admin
from feincms3 import plugins
from feincms3.admin import TreeAdmin

from testapp import models


@admin.register(models.Page)
class PageAdmin(ContentEditor, TreeAdmin):
    list_display = [
        "indented_title",
        "move_column",
        "is_active",
        "language_code",
        "page_type",
    ]
    list_filter = ["is_active"]
    list_editable = ["is_active"]
    prepopulated_fields = {"slug": ["title"]}
    raw_id_fields = ["parent"]
    inlines = [plugins.snippet.SnippetInline.create(model=models.Snippet)]
